package simFil;
/* Env ... store parameter, etc */

/*  <SimFil - simulation of cytoskeletal filament flexibility by Pairwise Agent Interaction with Rational Superposition (PAIRS)>
    Copyright (C) <2008>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.Color;
import java.awt.Font;
import java.io.File;
import java.util.Date;

import javax.vecmath.Color3f;
import javax.vecmath.Point3d;

import edu.cornell.lassp.houle.RngPack.RanMT;

public class Env { 
//  **** psuedo-random number generator (PRNG) ****
	static final RanMT mtRNG = new RanMT(new Date());
//  **** Useful Strings ***
	static final String kOnUnits = " /microMolar-s";
	static final String kOffUnits = " /s";
	static final String concUnits = " microMolar";
	static final String distUnits = " microns";
	static final String degUnits = " degrees";
	
//  **** Physical Constants ****
	static final double Boltz = 1.380662e-23;			// Boltzman's constant in J/K
	static final double tempK = 298.15;				// 25 deg Celsius in Kelvin
	static final double AvogadroNum = 6.02214199e23;	// Avogadro's number.. #/Mole
	
//	*** The Virtual World Dimensions ****
	static Color3f dumC = new Color3f(1.0f,0f,.2f);
	static final Pt3D worldDimension = new Pt3D(1,1,1);
	static final Pt3D farfarAway = new Pt3D(1000,1000,1000);
	
//	**** Times and Counters
	static private final double deltaT_init = 0.0001;		// seconds
	static double simulationTime = 0;			// keeps track of current time
	static private double runTime_init = 1200.0;				// final time to run to
	static int counter = 0;					// keeps track of current integration step number
	
	static final Parameter_ deltaT = new Parameter_ ("deltaT"," Time Step", deltaT_init, "seconds");
	static final Parameter_ runTime = new Parameter_ ("runTime"," Run Time", runTime_init, "seconds");
	
	
	//	**** Pairwise Agent Interactions with Rational Superposition (PAIRS) related ****
	static private double CTrans_init = 0.4;		// fraction of the calculated PAIRS force distance to move in a time-step... < 1 for stability
	static private double CsubR_init = 0.25;		// as above, but for torque from link force .. bigger numbers are stiffer / faster response times
	static private double CsubTheta_init = 0.0494;		// as above, but for torsion springs only.. bigger numbers are stiffer / faster response times
	
	static final Parameter_ CTrans = new Parameter_ ("CTrans"," PAIRS translation (C_delta)",CTrans_init,"");
	static final Parameter_ CsubR = new Parameter_ ("CsubR"," Torque arm for PAIRS translation (C_r)",CsubR_init,"");
	static final Parameter_ CsubTheta = new Parameter_ ("CsubTheta"," PAIRS torque (C_theta)",CsubTheta_init,"");
	
	//  **** Files and Paths ****
	static String outFileName = null;
	static String logFolderPath = null;
	static String srcFilePath = null;
	static String qkFilePath = null;
	static String fromQKFilePath;
	static String fromQKFileName;
	static String fromQKStatePath;
	static File paramFile = null;
	
//  **** For Synchronization of Methods ****
	static Object safeO = new Object();		// handle for operations that cannot be performed at the same time
	
//  **** Flags and Counters for Graphics ****
	static boolean remote = false;			// flag for no graphics instantiation
	static boolean logFiles = false;			// write info to files
	static boolean justLogLp = false; 		// flag used to write Lp angular correlations from GUI
	static boolean paintOn = true;
	static boolean toFile = false;
	static boolean toQKFile = false;
	static boolean fromQKFile = false;
	static boolean loadingQKFile = false;
	static boolean paused = true;
	static boolean viewRotation = false;
	static boolean orderedCentered = false;
	
	//static private boolean biochemOnly_init = false;		// flag for only biochemical simulation... no collisions, forces
	static final Parameter_ biochemOnly = new Parameter_ ("biochemOnly"," Biochem. Sim. Only", 0, " ", Parameter_.BOOLEAN,false);
	
	static private final int drawInterval_init = 100;
	static private final int toFileInterval_init = 100;
	static private final int toQKFileInterval_init = 100;
	static private final double jpegQuality_init = 1.0;
	static private final int remoteReportInterval_init = (int)1e6;		//time-steps
	static final boolean timeStampJPEGs = true;		// write time and other info on JPEG output
	
	static final Parameter_ drawInterval = new Parameter_ ("drawInterval"," Drawing Interval", drawInterval_init, "time steps",Parameter_.INT);
	static final Parameter_ toFileInterval = new Parameter_ ("toFileInterval"," JPEG Interval", toFileInterval_init, "time steps",Parameter_.INT);
	static final Parameter_ toQKFileInterval = new Parameter_ ("toQKFileInterval"," QK Interval", toQKFileInterval_init, "time steps",Parameter_.INT);
	static final Parameter_ jpegQuality = new Parameter_ ("jpegQuality"," JPEG Quality", jpegQuality_init, " (1.0 is best)");
	static final Parameter_ remoteReportInterval = new Parameter_ ("remoteWriteInterval", " Remote Reporting Interval",remoteReportInterval_init,"time steps",Parameter_.INT);
	
	static String toFilePath;
	static String toFileName;
	//static String toQKFilePath;
	//static String toQKFileName;
	
	// Rendering from QK files
	static boolean filRenderOff = false;
	static int curRenderNum = 0;
	
	//  **** Flags and Constants for Testing ****
	static boolean filBrownianMotionOff = false;
	static boolean plasmidBrownianMotionOff = false;
	static boolean testLinkedPlasmids = false;
	static final double tstPlasmidOffset = 0.2;
	static boolean compressionCritOff = false;
	
//  **** Graphics Sizes Etc ****
	static private final int frameWidth_init = 800;
	static private final int frameHeight_init = 460;
	
	static final Parameter_ frameWidth = new Parameter_ ("frameWidth"," Frame Width", frameWidth_init, "Pixels",Parameter_.INT);
	static final Parameter_ frameHeight = new Parameter_ ("frameHeight"," Frame Height", frameHeight_init, "Pixels",Parameter_.INT);
	
	static final int plasmidTessalation = 120;
	static final int parMEndSphereTessalation = 30;
	static final int monomerTessalation = 8;
	
	static boolean helixSpheres = true;
	static boolean filCoordSysOn = false;
	static boolean infoIn2D = true;
	
	static final double transScale_init = 7;
	static final Parameter_ transScale = new Parameter_ ("transScale"," Scale for 3D rendering", transScale_init, " ",Parameter_.DOUBLE);
	
	static final Parameter_ thickenFactor = new Parameter_ ("thickenFactor"," Thicken filament rendering factor", 1, " ",Parameter_.DOUBLE);
	
	static final int maxLayout = 30;

//	**** Initial Thing Creation ****
	static private final int initialFilaments_init = 0;
	static private final int initialPlasmids_init = 0;
	
	static final Parameter_ initialFilaments = new Parameter_ ("initialFilaments"," Initial ParM Filaments", initialFilaments_init," ",Parameter_.INT);
	static final Parameter_ initialPlasmids = new Parameter_ ("initialPlasmids"," Initial Plasmids", initialPlasmids_init," ",Parameter_.INT);
	
	
//  **** The Bug ****
	static boolean bugOn = false;
	static final double bugLength = 1.0;
	static final double bugRadius = 0.5;
	
	static private final double aeta_init = 0.1;			// viscosity in bug.... Pa-s (1e-3 is water)
	static final Parameter_ aeta = new Parameter_ ("aeta"," Viscosity in Bug", aeta_init, "Pa-s (1e-3 is water)");
	
//   **** The Spheres ****
	static final Parameter_ sphRadius = new Parameter_ ("sphRadius"," Plasmid Radius", 0.04, distUnits);
	
	static final Parameter_ sphTransDiff = new Parameter_ ("plasmidTransDiff"," Plasmid Translation Diffusivity", 1e-50, "m^2/s",Parameter_.DOUBLE);
	static final Parameter_ sphRotDiff = new Parameter_ ("plasmidRotDiff"," Plasmid Rotational Diffusivity", 1e-50, "/s",Parameter_.DOUBLE);
	
	static final Parameter_ sphCollisionCoeff = new Parameter_ ("sphCollisionCoeff"," PAIRS Coefficient for Sphere Collisions",0.4," ",Parameter_.DOUBLE);
	
	// values for small sphere between two planes
	static final Parameter_ midSphRadius = new Parameter_ ("midSphRadius"," Test Sphere Radius",0.02," microns",Parameter_.DOUBLE);
	static final Parameter_ wallSphRadius = new Parameter_ ("wallSphRadius"," Wall Spheres Radii",1.0," microns",Parameter_.DOUBLE);
	static final Parameter_ midSphXOffset = new Parameter_ ("midSphXOffset"," Test Sphere initial x-offset",0," microns",Parameter_.DOUBLE);
	static final Parameter_ midSphYOffset = new Parameter_ ("midSphYOffset"," Test Sphere initial y-offset",0.01," microns",Parameter_.DOUBLE);
	static final Parameter_ midSphZOffset = new Parameter_ ("midSphZOffset"," Test Sphere initial z-offset",0," microns",Parameter_.DOUBLE);
	
	// values for spheres squeezed into a container
	static final Parameter_ numSqueezed = new Parameter_ ("numSqueezed", " Number of spheres to squeeze", 10," ",Parameter_.INT);
	static final Parameter_ radSqueezed = new Parameter_ ("radSqueezed", " Radius of each squeezed sphere", 0.2," microns", Parameter_.DOUBLE);
	
	// values for spheres in force field
	static final Parameter_ numInBucket = new Parameter_ ("numInBucket", " Number of spheres in container", 10," ",Parameter_.INT);
	static final Parameter_ radInBucket = new Parameter_ ("radInBucket", " Radius of each sphere", 0.05," microns",Parameter_.DOUBLE);
	static final Parameter_ constantForce = new Parameter_ ("constantForce", " Constant force applied to each sphere", 1.0," pN",Parameter_.DOUBLE);
	
	// values for spheres centrally attracted
	static final Parameter_ numAttracted = new Parameter_ ("numAttracted", " Number of spheres attracted", 6," ", Parameter_.INT);
	static final Parameter_ radAttracted = new Parameter_ ("radAttracted", " Radius of each attracted sphere", 0.2," microns", Parameter_.DOUBLE);
	static final Parameter_ kAttracted = new Parameter_ ("kAttracted", " Spring constant of attraction to origin", 1.0," pN/nm", Parameter_.DOUBLE);
	
	//	 **** Filament Properties ****
	static final double persistenceLength_init = 15; // microns
	static final Parameter_ persistenceLength = new Parameter_ ("persistenceLength", " Persistence length", persistenceLength_init," microns", Parameter_.DOUBLE);	// in microns, use meters in defining EI below
	
	static final double hFromPlane_init = 0.5; // microns
	static final Parameter_ hFromPlane = new Parameter_ ("hFromPlane", " Dist. of filament from nearest surface", hFromPlane_init," microns", Parameter_.DOUBLE);
	
	static double EI = Boltz*tempK*(persistenceLength.getValue()*1e-6);		// EI = kTLp from worm-like chain theory
	
	static private final int stdSegLength_init = 100;							// number of monomers per segment
	static final Parameter_ stdSegLength = new Parameter_ ("filSegLength"," Std. segment length",stdSegLength_init," monomers",Parameter_.INT);
	
	static private final double monomerDiameter_init = 0.0049;					// microns
	static final Parameter_ monomerDiameter = new Parameter_ ("monomerDiameter"," Monomer Diameter",monomerDiameter_init," microns",Parameter_.DOUBLE);
	
	static private final double filamentRadius_init = 0.0035;					// microns
	static final Parameter_ filamentRadius = new Parameter_ ("filamentRadius"," Filament Radius",filamentRadius_init," microns",Parameter_.DOUBLE);
	
	static private final double filamentLength_init = 3.2;					// microns
	static final Parameter_ filamentLength = new Parameter_ ("filamentLength"," Filament Length",filamentLength_init," microns",Parameter_.DOUBLE);
	
	static final double helixWaveLength = 0.030;  	// (m) length for one turn of helical filament
	static final double monsPerHelixTurn = helixWaveLength/Env.monomerDiameter.getValue()/2;
	static final double helixAngInc = 2*Math.PI/Env.monsPerHelixTurn;
	static final double helixPitch = 2*Math.PI/helixWaveLength;
	static final double helixMonOffset = (Env.filamentRadius.getValue()-Env.monomerDiameter.getValue())/2;
	
	static private final double filTorqSpring_init = 1.0e-20;					// N/radian   If using elastic springs for torque to keep filaments aligned
	static final Parameter_ filTorqSpring = new Parameter_ ("filTorqSpring"," Filament Torsional Spring",filTorqSpring_init," N/radian",Parameter_.DOUBLE,false);
	
	static private double BTransCoeff_init = 1.1;		// range from zero (no brownian motion) to infinity
	static final Parameter_ BTransCoeff = new Parameter_ ("BTransCoeff"," Brownian Translational (B_t)",BTransCoeff_init,"");
	
	static private double BRotCoeff_init = 0.5;		// range from zero (no brownian motion) to infinity
	static final Parameter_ BRotCoeff = new Parameter_ ("BRotCoeff"," Brownian Rotational (B_theta)",BRotCoeff_init,"");
	
	static final Parameter_ tracForce = new Parameter_ ("tracForce"," Traction force applied to filament ends",1.0," pN",Parameter_.DOUBLE);
	
	//  **** Attachment to plasmid
	static final Parameter_ end1CanBindPlasmid = new Parameter_ ("end1CanBindPlasmid"," Can end1 of segment bind plasmid?",0," ",Parameter_.BOOLEAN,true);
	static final Parameter_ end2CanBindPlasmid = new Parameter_ ("end2CanBindPlasmid"," Can end2 of segment bind plasmid?",0," ",Parameter_.BOOLEAN,true);
	static final double plasmidBoundTransScale = 0.1;
	
	static private final double plasmidTorqSpring_init = 9.0e-20;				// N/radian  If using elastic spring to keep filament orthogonal to plasmid surface
	static final Parameter_ plasmidTorqSpring = new Parameter_ ("plasmidTorqSpring"," Cantilever support spring",plasmidTorqSpring_init,"N/radian",Parameter_.DOUBLE);
	

//   **** Show Stats Parameters ****
	static final Parameter_ showTime = new Parameter_ ("showTime"," Show Time",0,"",Parameter_.BOOLEAN);
	static final Parameter_ showConc = new Parameter_ ("showConc"," Show ParM Conc",0,"",Parameter_.BOOLEAN);
	static final Parameter_ showNonHydroConc = new Parameter_ ("showNonHydroConc"," Show Non-Hydro ParM Conc",0,"",Parameter_.BOOLEAN,false);
	static final Parameter_ showFilCt = new Parameter_ ("showFilCt"," Show Filament Count",0,"",Parameter_.BOOLEAN,false);
	static final Parameter_ showFilLinkCt = new Parameter_ ("showFilLinkCt"," Show FilSegment Link Count",0,"",Parameter_.BOOLEAN,false);
	static final Parameter_ showFilSegCt = new Parameter_ ("showFilSegCt"," Show FilSegment Count",0,"",Parameter_.BOOLEAN,false);
	static final Parameter_ showMonCt = new Parameter_ ("showMonCt"," Show Monomer Count",0,"",Parameter_.BOOLEAN);
	
//	 **** Filament Length Histogram Parameters ****
	static private final double rStart_init = 0;	// (micron) start of range for tracking filament lengths
	static private final double rStop_init = 1;	// (micron) end of range for tracking filament lengths
	static private final int binCt_init = 100;		// number of bins in histogram
	static private final int histWidth_init = 200;
	static private final int histHeight_init = 200;
	static private final int histX_init = 0;
	static private final int histY_init = 0;
	static private final int fixedHistScale_init = 100;	// default value using fixed vertical scale in filament length histogram
	
	
	static final Parameter_ makeFilLengthHisto = new Parameter_ ("makeFilLengthHisto"," Filament Length Histogram?",0,"",Parameter_.BOOLEAN,false);
	static final Parameter_ mapMonomerCtToHisto = new Parameter_ ("mapMonomerCtToHisto"," Map Monomer Count Instead",0,"",Parameter_.BOOLEAN);
	static final Parameter_ rStart = new Parameter_ ("rStart"," Starting Filament Length",rStart_init,distUnits);
	static final Parameter_ rStop = new Parameter_ ("rStop"," Ending Filament Length",rStop_init,distUnits);
	static final Parameter_ binCt = new Parameter_ ("binCt"," Bins in Histogram",binCt_init,"",Parameter_.INT);
	static final Parameter_ fixedHistScale = new Parameter_ ("fixedHistScale"," Fixed Vertical Scale",fixedHistScale_init," filaments",Parameter_.DOUBLE,false);
	static final Parameter_ keepHeads = new Parameter_ ("keepHeads"," Low Values in First Bin",0,"",Parameter_.BOOLEAN);
	static final Parameter_ keepTails = new Parameter_ ("keepTails"," High Values in Last Bin",0,"",Parameter_.BOOLEAN);
	static final Parameter_ histWidth = new Parameter_ ("histWidth"," Width",histWidth_init,"pixels",Parameter_.INT);
	static final Parameter_ histHeight = new Parameter_ ("histHeight"," Height",histHeight_init,"pixels",Parameter_.INT);
	static final Parameter_ histX = new Parameter_ ("histX"," X-Position",histX_init,"pixels",Parameter_.INT);
	static final Parameter_ histY = new Parameter_ ("histY"," Y-Position",histY_init,"pixels",Parameter_.INT);
	static final Parameter_ showHistBox = new Parameter_ ("showHistBox"," Show Box",0,"",Parameter_.BOOLEAN);
	static final Parameter_ showYHistTics = new Parameter_ ("showYHistTics"," Show Verical Scale",0,"",Parameter_.BOOLEAN);
	static final Parameter_ flatTops = new Parameter_ ("flatTops"," Flat Tops on Bins",0,"",Parameter_.BOOLEAN);
	
	// for persistence length tuning
	static boolean testingPersistenceLength = false;
	static int ckPersistenceInterval = (int)2e5; 
	static int persistenceValsToTrack = 100000;
	static double persistenceTstLengthInit = 3.2;	// microns.... approximate desired length
	static int persistenceSegNumber = (int)(persistenceTstLengthInit/(Env.stdSegLength.getValue()*Env.monomerDiameter.getValue()/2)); 
	static double persistenceTstLength = persistenceSegNumber*Env.stdSegLength.getValue()*Env.monomerDiameter.getValue()/2;
	static ValueTracker_ [] persistenceTrack;
	static ValueTracker_ transDiffTrack = new ValueTracker_(persistenceValsToTrack,ValueTracker_.PT3D_TYPE);
	static Pt3D lastPos = new Pt3D();
	
	// tuning options
	static boolean cantileverElasticityTst = false;
	static boolean simpleSupportElasticityTst = true;
	static boolean persistenceTst = false;
	static boolean filUnderTraction = false;
	static boolean sphereBetweenPlanes = false;
	static boolean spheresUnderForceField = false;
	static boolean spheresSqueezeFit = false;
	static boolean spheresLinked = false;
	
	//	 for elasticity tuning
	static boolean secondMode = false;	
	static boolean relaxCantilever = false;
	static boolean relaxSimpleSupport = false;
	static int ckElasticityInterval = (int)1e6;  // 1 seconds with deltaT of 1e-5
	static double ckElasticitySmallChange = 0.0001;  // .1 nm ... if value hasn't changed by 1 nm over ckElasticityInterval. then we have the value
	static double initDeltaE = 1e-19;
	static double curDeltaE = initDeltaE;
	static double lastDelta = 0;
	static double curDeltaCtheta = 0.1;
	static double deltaTolerance = 1e-8;
	static int deltaTrackCounter = 0;
	static int deltaTrackInt = (int)1e4;
	static boolean lastSwitchPlus = false;
	static boolean goingPlus = false;
	static boolean shouldGoPlus = false;
	
	static boolean lastEChangePlus = true;
	static double valOKChange = 0.0001;
	static double tstFilLengthInit = 3.2;//3.2; // microns	This is the approximate desired starting length
	static double tstFilLengthMax = 5.0; // microns   	the approximate end length
	static int curMultipleOfSeg; 
	static int forceSeg1,forceSeg2;		// seg filArrayPos for application of force
	static double desiredDeflection = -0.1;  // microns
	static double forceToGetDesiredDeflection;// = 3*EI*(desiredDeflection*1e-6)/Math.pow(tstFilLength*1e-6,3);
	static Pt3D staticForce = new Pt3D (0,0,0);	// force only applied in y-direction
	static Pt3D staticForceR = new Pt3D (0,0,0);
	static Pt3D rToCross = new Pt3D ();					// used in elasticity torque calc.
	static double expectedDeflection; // = (staticForce.y*Math.pow(tstFilLength*1e-6,3)/(3*EI))*1e6;	// in microns.. from beam theory y=FL^3/(3EI)
	static double relaxationDeflection;
	static double currentDeflection = 0;
	static double lastDeflection = 0;
	static double curDeltaD = 0;
	static double curDistFromExpected = 0;
	static double lastDistFromExpected = 1e6;
	static String elasReportString = " ";
	static int relaxWriteInterval = (int)1e2;
	static int relaxWriteCounter = relaxWriteInterval;  // so it'll write right away when supports relaxed
	static boolean reportedTimeConstant = false;
	
//	 **** 3D Quality ****
	static int graphicsQuality = 0;	// specify graphics quality
	static final int LOW_QUALITY = 0;
	static final int MID_QUALITY = 1;
	static final int HIGH_QUALITY = 2;
	
	static final int lowSphereTessalation = 10;	// number of polygons to define a sphere
	static final int midSphereTessalation = 18;
	static final int highSphereTessalation = 24;
	
	static final int lowCylXTessalation = 10;		// x and y resolution for cylinders
	static final int lowCylYTessalation = 10;
	static final int midCylXTessalation = 18;
	static final int midCylYTessalation = 18;
	static final int highCylXTessalation = 24;
	static final int highCylYTessalation = 24;
	
	// filament appearance
	static boolean filamentLines = true;
	static boolean filamentCyls = false;
	
	// monomer appearance
	static boolean eachMonomerASphere = false;
	static boolean eachMonomerADot = false;
	
//	 **** GUI AND COLOR RELATED ****
	/* // gray colors
	static final Color3f universeColor3f = new Color3f(0.6f,0.6f,0.6f);
	static final Color universeColor = universeColor3f.get();
	static final Color3f headlightColor = new Color3f(1.0f,1.0f,1.0f);
	static final Color3f cellColor3f = new Color3f(0.1f,0.1f,0.1f);
	static final Color3f plasmidColor3f = new Color3f(1.0f,1.0f,1.0f);
	static final Color3f parRparCColor3f = new Color3f(0f,1.0f,1.0f);
	static final Color cellColor = cellColor3f.get();
	static final Color cellShadeColor = Color.BLACK;
	static final Font controlFont = new Font(null,Font.PLAIN,10);
	static final Font headFont = new Font(null,Font.BOLD,10);
	static final Font elasticityFont = new Font(null,Font.BOLD,12);
	static final Color3f controlBackColor3f = new Color3f(0.6f,0.6f,0.6f);
	static final Color controlBackColor = controlBackColor3f.get();//Color.BLACK;
	static final Color controlForeColor = Color.BLACK;//Color.WHITE;
	static final Color controlFlashColor = Color.WHITE;
	static final Color canvasTextColor = new Color(0.0f,0.0f,0.0f);
	static final Color3f bugEndLightColor3f = new Color3f(0.8f,0.8f,0.8f);
	static final Color3f plasmidLightColor3f = new Color3f(0.0f,0.5f,1.0f);*/
	
	/*// white colors
	static final Color3f universeColor3f = new Color3f(0.9f,0.9f,0.9f);
	static final Color universeColor = universeColor3f.get();
	static final Color3f cellColor3f = new Color3f(0.1f,0.1f,0.1f);
	static final Color3f plasmidColor3f = new Color3f(1.0f,1.0f,1.0f);
	static final Color3f parRparCColor3f = new Color3f(0f,1.0f,1.0f);
	static final Color cellColor = cellColor3f.get();
	static final Color cellShadeColor = Color.BLACK;
	static final Font controlFont = new Font(null,Font.PLAIN,10);
	static final Font headFont = new Font(null,Font.BOLD,10);
	static final Font buttonFont = new Font(null,Font.BOLD,14);
	static final Font elasticityFont = new Font(null,Font.BOLD,12);
	static final Color3f controlBackColor3f = new Color3f(0.6f,0.6f,0.6f);
	static final Color controlBackColor = controlBackColor3f.get();//Color.BLACK;
	static final Color controlForeColor = Color.BLACK;//Color.WHITE;
	static final Color buttonForeColor = Color.BLUE;
	static final Color buttonBackColor = controlBackColor;
	static final Color controlFlashColor = Color.WHITE;
	static final Color canvasTextColor = new Color(0.0f,0.0f,0.0f);
	static final Color3f headlightColor = new Color3f(0.8f,0.9f,0.8f);
	static final Color3f spotLightColor3f = new Color3f(1.0f,1.0f,1.0f);
	static final Color3f bugEndLightColor3f = new Color3f(0.8f,0.8f,0.8f);
	static final Color3f plasmidLightColor3f = new Color3f(0.0f,0.4f,1.0f);
	*/
	
	// black colors
	static final Color3f universeColor3f = new Color3f(0.0f,0.0f,0.0f);
	static final Color universeColor = universeColor3f.get();
	static final Color3f headlightColor = new Color3f(1.0f,1.0f,1.0f);
	static final Color3f cellColor3f = new Color3f(0.5f,0.5f,0.5f);
	static final Color3f plasmidColor3f = new Color3f(1.0f,1.0f,1.0f);
	static final Color3f parRparCColor3f = new Color3f(0.0f,1.0f,1.0f);
	static final Font controlFont = new Font(null,Font.PLAIN,10);
	static final Font headFont = new Font(null,Font.BOLD,10);
	static final Font buttonFont = new Font(null,Font.BOLD,14);
	static final Font elasticityFont = new Font(null,Font.BOLD,12);
	static final Color3f controlBackColor3f = new Color3f(0.0f,0.0f,0.0f);
	static final Color controlBackColor = controlBackColor3f.get();//Color.BLACK;
	static final Color controlForeColor = new Color(0.8f,0.8f,1.0f);
	static final Color buttonForeColor = Color.BLUE;
	static final Color buttonBackColor = controlBackColor;
	static final Color canvasTextColor = new Color(0.9f,0.9f,0.9f);
	static final Color3f bugEndLightColor3f = new Color3f(0.8f,0.8f,0.8f);
	static final Color3f plasmidLightColor3f = new Color3f(0.0f,0.6f,1.0f);
	
	
	public static void resetElasticityTstValues () {
		EI = Boltz*tempK*(persistenceLength.getValue()*1e-6);		// EI = kTLp from worm-like chain theory
		Env.reportedTimeConstant = false;
		Env.relaxSimpleSupport = false;
		Env.relaxCantilever = false;
		FilSegment.releaseAllPlasmids = false;
		curMultipleOfSeg = (int)(Env.filamentLength.getValue()/(Env.stdSegLength.getValue()*Env.monomerDiameter.getValue()/2));
		Env.filamentLength.setValue((curMultipleOfSeg*Env.stdSegLength.getValue()+1)*Env.monomerDiameter.getValue()/2);
		if (cantileverElasticityTst) {
			desiredDeflection = -0.01*filamentLength.getValue();
			forceToGetDesiredDeflection = 3*EI*(desiredDeflection*1e-6)/Math.pow(filamentLength.getValue()*1e-6,3);
			expectedDeflection = (forceToGetDesiredDeflection*Math.pow(filamentLength.getValue()*1e-6,3)/(3*EI))*1e6;	// in microns.. from beam theory y=FL^3/(3EI)
		} else {
			desiredDeflection = -0.01*filamentLength.getValue();
			forceToGetDesiredDeflection = 48*EI*(desiredDeflection*1e-6)/Math.pow(filamentLength.getValue()*1e-6,3);
			expectedDeflection = (forceToGetDesiredDeflection*Math.pow(filamentLength.getValue()*1e-6,3)/(48*EI))*1e6;	// in microns.. from beam theory y=FL^3/(48EI)
			if (secondMode) {
				expectedDeflection = 0;
				forceSeg1 = Env.curMultipleOfSeg/4 + 1;
				forceSeg2 = 3*Env.curMultipleOfSeg/4 + 1;
			} else {
				forceSeg1 = Env.curMultipleOfSeg/2 + 1;
			}
			//System.out.println ("forceSegs are " + forceSeg1 + ", " + forceSeg2);
		}
		staticForce.y = forceToGetDesiredDeflection;	// force only applied in y-direction
		//System.out.println ("Static force in y is " + staticForce.y);
		staticForceR.y = -forceToGetDesiredDeflection;
		curDeltaE = initDeltaE;
		lastDistFromExpected = 1e6;
	}
	
	public static void resetPersistenceTstValues () {
		EI = Boltz*tempK*(persistenceLength.getValue()*1e-6);		// EI = kTLp from worm-like chain theory
		Env.relaxSimpleSupport = false;
		Env.relaxCantilever = false;
		curMultipleOfSeg = (int)(Env.filamentLength.getValue()/(Env.stdSegLength.getValue()*Env.monomerDiameter.getValue()/2));
		Env.filamentLength.setValue((curMultipleOfSeg*Env.stdSegLength.getValue()+1)*Env.monomerDiameter.getValue()/2);
		persistenceSegNumber = curMultipleOfSeg;
		persistenceTstLength = Env.filamentLength.getValue();
		persistenceTrack = new ValueTracker_ [Env.persistenceSegNumber];
		for (int i=0;i<persistenceSegNumber;i++) {
			persistenceTrack[i] = new ValueTracker_(persistenceValsToTrack,ValueTracker_.DOUBLE_TYPE);
		}
		transDiffTrack.zero();
	}
	
}

